@@ -33,6 +33,7 @@
 #include "peas-object-module.h"
 #include "peas-extension.h"
 #include "peas-dirs.h"
+#include "peas-debug.h"
 
 /**
  * SECTION:peas-engine
@@ -77,14 +78,12 @@ struct _PeasEnginePrivate {
 
   GList *plugin_list;
   GHashTable *loaders;
-
-  GList *object_list;
 };
 
 static void peas_engine_load_plugin_real   (PeasEngine     *engine,
-                                            PeasPluginInfo * info);
-static void peas_engine_unload_plugin_real (PeasEngine     * engine,
-                                            PeasPluginInfo * info);
+                                            PeasPluginInfo *info);
+static void peas_engine_unload_plugin_real (PeasEngine     *engine,
+                                            PeasPluginInfo *info);
 
 static void
 load_plugin_info (PeasEngine  *engine,
@@ -117,7 +116,7 @@ load_dir_real (PeasEngine  *engine,
                const gchar *extension,
                const gchar *module_dir,
                const gchar *data_dir,
-               unsigned int recursions)
+               guint        recursions)
 {
   GError *error = NULL;
   GDir *d;
@@ -129,7 +128,7 @@ load_dir_real (PeasEngine  *engine,
 
   if (!d)
     {
-      g_warning ("%s", error->message);
+      g_message ("%s", error->message);
       g_error_free (error);
       return;
     }
@@ -163,19 +162,25 @@ load_dir_real (PeasEngine  *engine,
 void
 peas_engine_rescan_plugins (PeasEngine *engine)
 {
+  gchar **sp;
   gchar *extension;
   guint i;
-  gchar **sp;
 
   g_return_if_fail (PEAS_IS_ENGINE (engine));
 
+  sp = engine->priv->search_paths;
+  if (sp == NULL)
+    {
+      g_debug ("No search paths where provided");
+      return;
+    }
+
   /* Compute the extension of the plugin files. */
   extension = g_strdup_printf (".%s-plugin", engine->priv->app_name);
   for (i = 0; extension[i] != '\0'; ++i)
     extension[i] = g_ascii_tolower (extension[i]);
 
   /* Go and read everything from the provided search paths */
-  sp = engine->priv->search_paths;
   for (i = 0; sp[i] != NULL; i += 2)
     load_dir_real (engine, extension, sp[i], sp[i + 1], 1);
 
@@ -196,10 +201,10 @@ hash_lowercase (gconstpointer data)
 }
 
 static gboolean
-equal_lowercase (gconstpointer a,
-                 gconstpointer b)
+equal_lowercase (const gchar *a,
+                 const gchar *b)
 {
-  return g_ascii_strcasecmp ((const gchar *) a, (const gchar *) b) == 0;
+  return g_ascii_strcasecmp (a, b) == 0;
 }
 
 static void
@@ -243,11 +248,9 @@ peas_engine_init (PeasEngine *engine)
                                               PEAS_TYPE_ENGINE,
                                               PeasEnginePrivate);
 
-  engine->priv->object_list = NULL;
-
   /* mapping from loadername -> loader object */
   engine->priv->loaders = g_hash_table_new_full (hash_lowercase,
-                                                 equal_lowercase,
+                                                 (GEqualFunc) equal_lowercase,
                                                  (GDestroyNotify) g_free,
                                                  (GDestroyNotify) loader_destroy);
 }
@@ -309,9 +312,9 @@ peas_engine_set_property (GObject      *object,
 }
 
 static void
-peas_engine_get_property (GObject   *object,
-                          guint      prop_id,
-                          GValue    *value,
+peas_engine_get_property (GObject    *object,
+                          guint       prop_id,
+                          GValue     *value,
                           GParamSpec *pspec)
 {
   PeasEngine *engine = PEAS_ENGINE (object);
@@ -356,7 +359,6 @@ peas_engine_finalize (GObject *object)
     _peas_plugin_info_unref (PEAS_PLUGIN_INFO (item->data));
 
   g_list_free (engine->priv->plugin_list);
-  g_list_free (engine->priv->object_list);
   g_strfreev (engine->priv->search_paths);
   g_free (engine->priv->app_name);
   g_free (engine->priv->base_module_dir);
@@ -374,6 +376,7 @@ peas_engine_class_init (PeasEngineClass *klass)
   object_class->get_property = peas_engine_get_property;
   object_class->constructed = peas_engine_constructed;
   object_class->finalize = peas_engine_finalize;
+
   klass->load_plugin = peas_engine_load_plugin_real;
   klass->unload_plugin = peas_engine_unload_plugin_real;
 
@@ -387,7 +390,8 @@ peas_engine_class_init (PeasEngineClass *klass)
    * the .gedit-plugin extension, and the engine will look for a
    * "Gedit Plugin" section in it to load the plugin data.
    */
-  g_object_class_install_property (object_class, PROP_APP_NAME,
+  g_object_class_install_property (object_class,
+                                   PROP_APP_NAME,
                                    g_param_spec_string ("app-name",
                                                         "Application Name",
                                                         "The application name",
@@ -405,7 +409,8 @@ peas_engine_class_init (PeasEngineClass *klass)
    * the base module directory. For instance, the python loader will
    * look for python modules in "${base-module-dir}/python/".
    */
-  g_object_class_install_property (object_class, PROP_BASE_MODULE_DIR,
+  g_object_class_install_property (object_class,
+                                   PROP_BASE_MODULE_DIR,
                                    g_param_spec_string ("base-module-dir",
                                                         "Base module dir",
                                                         "The base application dir for binding modules lookup",
@@ -446,7 +451,8 @@ peas_engine_class_init (PeasEngineClass *klass)
    * };
    * ]|
    */
-  g_object_class_install_property (object_class, PROP_SEARCH_PATHS,
+  g_object_class_install_property (object_class,
+                                   PROP_SEARCH_PATHS,
                                    g_param_spec_boxed ("search-paths",
                                                        "Search paths",
                                                        "The list of paths where to look for plugins",
@@ -493,6 +499,10 @@ peas_engine_class_init (PeasEngineClass *klass)
                   G_SIGNAL_TYPE_STATIC_SCOPE);
 
   g_type_class_add_private (klass, sizeof (PeasEnginePrivate));
+
+  /* We are doing some global initialization here as there is currently no
+   * global init function for libpeas. */
+  peas_debug_init ();
 }
 
 static LoaderInfo *
@@ -529,7 +539,8 @@ load_plugin_loader (PeasEngine  *engine,
 
       if (loader == NULL || !PEAS_IS_PLUGIN_LOADER (loader))
         {
-          g_warning ("Loader object is not a valid PeasPluginLoader instance");
+          g_warning ("Loader '%s' is not a valid PeasPluginLoader instance",
+                     loader_id);
           if (loader != NULL && G_IS_OBJECT (loader))
             g_object_unref (loader);
           module = NULL;
@@ -546,7 +557,7 @@ load_plugin_loader (PeasEngine  *engine,
     }
   else
     {
-      g_warning ("Plugin loader module `%s' could not be loaded",
+      g_warning ("Loader '%s' could not be loaded",
                  loader_id);
       g_object_unref (module);
       module = NULL;
@@ -601,7 +612,8 @@ peas_engine_disable_loader (PeasEngine  *engine,
                                                     loader_id);
   if (loader_info != NULL)
     {
-      g_warning ("Loader %s cannot be disabled as it is already loaded", loader_id);
+      g_warning ("Loader '%s' cannot be disabled as it is already loaded",
+                 loader_id);
       return;
     }
 
@@ -616,8 +628,9 @@ peas_engine_disable_loader (PeasEngine  *engine,
  *
  * Returns the list of #PeasPluginInfo known to the engine.
  *
- * Returns: a #GList of #PeasPluginInfo. Note that the list
- * belongs to the engine and should not be freed.
+ * Returns: (transfer none) (element-type Peas.PluginInfo): a #GList of
+ * #PeasPluginInfo. Note that the list belongs to the engine and should
+ * not be freed.
  **/
 const GList *
 peas_engine_get_plugin_list (PeasEngine *engine)
@@ -666,7 +679,7 @@ load_plugin (PeasEngine     *engine,
 
   if (loader == NULL)
     {
-      g_warning ("Could not find loader `%s' for plugin `%s'",
+      g_warning ("Could not find loader '%s' for plugin '%s'",
                  info->loader, info->name);
       info->available = FALSE;
       return FALSE;
@@ -760,9 +773,9 @@ peas_engine_unload_plugin (PeasEngine     *engine,
 }
 
 gboolean
-peas_engine_provides_extension (PeasEngine *engine,
+peas_engine_provides_extension (PeasEngine     *engine,
                                 PeasPluginInfo *info,
-                                GType extension_type)
+                                GType           extension_type)
 {
   PeasPluginLoader *loader;
 
@@ -777,9 +790,9 @@ peas_engine_provides_extension (PeasEngine *engine,
 }
 
 PeasExtension *
-peas_engine_get_extension (PeasEngine *engine,
+peas_engine_get_extension (PeasEngine     *engine,
                            PeasPluginInfo *info,
-                           GType extension_type)
+                           GType           extension_type)
 {
   PeasPluginLoader *loader;
 
@@ -835,7 +848,7 @@ string_in_strv (const gchar  *needle,
 /**
  * peas_engine_set_loaded_plugins:
  * @engine: A #PeasEngine.
- * @plugin_names: A NULL-terminated array of plugin names.
+ * @plugin_names: A %NULL-terminated array of plugin names.
  *
  * Sets the list of loaded plugins for @engine. When this function is called,
  * the #PeasEngine will load all the plugins whose names are in @plugin_names,
@@ -871,7 +884,7 @@ peas_engine_set_loaded_plugins (PeasEngine   *engine,
 
 /**
  * peas_engine_new:
- * @app_name: The name of the app
+ * @app_name: (allow-none): The name of the application
  * @base_module_dir: The base directory for language modules
  * @search_paths: The paths where to look for plugins
  *
@@ -885,6 +898,9 @@ peas_engine_new (const gchar  *app_name,
                  const gchar  *base_module_dir,
                  const gchar **search_paths)
 {
+  g_return_val_if_fail (base_module_dir != NULL, NULL);
+  g_return_val_if_fail (search_paths != NULL, NULL);
+
   return PEAS_ENGINE (g_object_new (PEAS_TYPE_ENGINE,
                                     "app-name", app_name,
                                     "base-module-dir", base_module_dir,
