@@ -161,22 +161,21 @@ about_button_cb (GtkWidget           *button,
   PeasPluginInfo *info;
 
   info = plugin_manager_get_selected_plugin (pm);
-
   g_return_if_fail (info != NULL);
 
   /* if there is another about dialog already open destroy it */
   if (pm->priv->about)
     gtk_widget_destroy (pm->priv->about);
 
-  pm->priv->about = g_object_new (GTK_TYPE_ABOUT_DIALOG,
-                                  "program-name", peas_plugin_info_get_name (info),
-                                  "copyright", peas_plugin_info_get_copyright (info),
-                                  "authors", peas_plugin_info_get_authors (info),
-                                  "comments", peas_plugin_info_get_description (info),
-                                  "website", peas_plugin_info_get_website (info),
-                                  "logo-icon-name", peas_ui_plugin_info_get_icon_name (info),
-                                  "version", peas_plugin_info_get_version (info),
-                                  NULL);
+  pm->priv->about = GTK_WIDGET (g_object_new (GTK_TYPE_ABOUT_DIALOG,
+                                              "program-name", peas_plugin_info_get_name (info),
+                                              "copyright", peas_plugin_info_get_copyright (info),
+                                              "authors", peas_plugin_info_get_authors (info),
+                                              "comments", peas_plugin_info_get_description (info),
+                                              "website", peas_plugin_info_get_website (info),
+                                              "logo-icon-name", peas_ui_plugin_info_get_icon_name (info),
+                                              "version", peas_plugin_info_get_version (info),
+                                              NULL));
 
   gtk_window_set_destroy_with_parent (GTK_WINDOW (pm->priv->about), TRUE);
 
@@ -201,7 +200,9 @@ configure_button_cb (GtkWidget           *button,
   PeasPluginInfo *info;
   PeasExtension *exten;
   GtkWindow *toplevel;
-  GtkWidget *conf_dlg = NULL;
+  GtkWidget *conf_widget = NULL;
+  GtkWidget *conf_dlg;
+  GtkWidget *vbox;
   GtkWindowGroup *wg;
 
   info = plugin_manager_get_selected_plugin (pm);
@@ -210,17 +211,30 @@ configure_button_cb (GtkWidget           *button,
   exten = peas_engine_get_extension (pm->priv->engine, info, PEAS_UI_TYPE_CONFIGURABLE); 
   g_return_if_fail (exten != NULL);
 
-  g_debug ("Calling create_configure_dialog on %p", exten);
+  peas_extension_call (exten, "create_configure_widget", &conf_widget);
 
-  peas_extension_call (exten, "create_configure_dialog", &conf_dlg);
   g_object_unref (exten);
-  g_return_if_fail (conf_dlg != NULL);
+  g_return_if_fail (GTK_IS_WIDGET (conf_widget));
+  g_return_if_fail (!gtk_widget_is_toplevel (conf_widget));
 
   toplevel = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (pm)));
-  gtk_window_set_transient_for (GTK_WINDOW (conf_dlg), toplevel);
 
-  wg = gtk_window_get_group (toplevel);
-  if (wg == NULL)
+  conf_dlg = gtk_dialog_new_with_buttons (peas_plugin_info_get_name (info),
+                                          toplevel,
+                                          GTK_DIALOG_NO_SEPARATOR,
+                                          GTK_STOCK_CLOSE,
+                                          GTK_RESPONSE_CLOSE,
+                                          NULL);
+  g_signal_connect (conf_dlg, "response", G_CALLBACK (gtk_widget_destroy), NULL);
+
+  vbox = gtk_dialog_get_content_area (GTK_DIALOG (conf_dlg));
+  gtk_box_pack_start (GTK_BOX (vbox), conf_widget, TRUE, TRUE, 0);
+
+  if (gtk_window_has_group (toplevel))
+    {
+      wg = gtk_window_get_group (toplevel);
+    }
+  else
     {
       wg = gtk_window_group_new ();
       gtk_window_group_add_window (wg, toplevel);
@@ -228,6 +242,7 @@ configure_button_cb (GtkWidget           *button,
 
   gtk_window_group_add_window (wg, GTK_WINDOW (conf_dlg));
 
+  gtk_window_set_transient_for (GTK_WINDOW (conf_dlg), toplevel);
   gtk_window_set_modal (GTK_WINDOW (conf_dlg), TRUE);
   gtk_widget_show_all (conf_dlg);
 }
@@ -299,9 +314,7 @@ active_toggled_cb (GtkCellRendererToggle *cell,
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (pm->priv->tree));
   g_return_if_fail (model != NULL);
 
-  gtk_tree_model_get_iter (model, &iter, path);
-
-  if (&iter != NULL)
+  if (gtk_tree_model_get_iter (model, &iter, path))
     plugin_manager_toggle_active (pm, &iter, model);
 
   gtk_tree_path_free (path);
@@ -331,14 +344,10 @@ row_activated_cb (GtkTreeView         *tree_view,
   GtkTreeModel *model;
 
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (pm->priv->tree));
-
   g_return_if_fail (model != NULL);
 
-  gtk_tree_model_get_iter (model, &iter, path);
-
-  g_return_if_fail (&iter != NULL);
-
-  plugin_manager_toggle_active (pm, &iter, model);
+  if (gtk_tree_model_get_iter (model, &iter, path))
+    plugin_manager_toggle_active (pm, &iter, model);
 }
 
 static void
@@ -375,8 +384,7 @@ plugin_manager_populate_lists (PeasUIPluginManager *pm)
       GtkTreeSelection *selection;
       PeasPluginInfo *info;
 
-      selection =
-        gtk_tree_view_get_selection (GTK_TREE_VIEW (pm->priv->tree));
+      selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (pm->priv->tree));
       g_return_if_fail (selection != NULL);
 
       gtk_tree_selection_select_iter (selection, &iter);
@@ -399,7 +407,6 @@ plugin_manager_set_active (PeasUIPluginManager *pm,
   gboolean res = TRUE;
 
   gtk_tree_model_get (model, iter, INFO_COLUMN, &info, -1);
-
   g_return_val_if_fail (info != NULL, FALSE);
 
   if (active)
@@ -463,10 +470,10 @@ plugin_manager_set_active_all (PeasUIPluginManager *pm,
   GtkTreeIter iter;
 
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (pm->priv->tree));
-
   g_return_if_fail (model != NULL);
 
-  gtk_tree_model_get_iter_first (model, &iter);
+  if (!gtk_tree_model_get_iter_first (model, &iter))
+    return;
 
   do
     plugin_manager_set_active (pm, &iter, model, active);
@@ -555,6 +562,9 @@ create_tree_popup_menu (PeasUIPluginManager *pm)
 
   info = plugin_manager_get_selected_plugin (pm);
 
+  if (info == NULL)
+    return NULL;
+
   menu = gtk_menu_new ();
 
   item = gtk_image_menu_item_new_with_mnemonic (_("_About"));
@@ -673,6 +683,9 @@ show_tree_popup_menu (GtkTreeView         *tree,
 
   pm->priv->popup_menu = create_tree_popup_menu (pm);
 
+  if (pm->priv->popup_menu == NULL)
+    return;
+
   gtk_menu_attach_to_widget (GTK_MENU (pm->priv->popup_menu),
                              GTK_WIDGET (pm),
                              (GtkMenuDetachFunc) tree_popup_menu_detach);
@@ -839,10 +852,8 @@ plugin_toggled_cb (PeasEngine          *engine,
       info_found = info == tinfo;
     }
 
-  if (!info_found)
+  if (!info_found && gtk_tree_model_get_iter_first (model, &iter))
     {
-      gtk_tree_model_get_iter_first (model, &iter);
-
       do
         {
           PeasPluginInfo *tinfo;
@@ -896,7 +907,7 @@ peas_ui_plugin_manager_init (PeasUIPluginManager *pm)
   hbuttonbox = gtk_hbutton_box_new ();
   gtk_box_pack_start (GTK_BOX (pm), hbuttonbox, FALSE, FALSE, 0);
   gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_END);
-  gtk_box_set_spacing (GTK_BOX (hbuttonbox), 8);
+  gtk_box_set_spacing (GTK_BOX (hbuttonbox), 8); /* should use the default or HIG */
 
   pm->priv->about_button = gtk_button_new_from_stock (GTK_STOCK_ABOUT);
   gtk_container_add (GTK_CONTAINER (hbuttonbox), pm->priv->about_button);
@@ -938,7 +949,6 @@ peas_ui_plugin_manager_constructed (GObject *object)
       gtk_widget_set_sensitive (pm->priv->about_button, FALSE);
       gtk_widget_set_sensitive (pm->priv->configure_button, FALSE);
     }
-
 }
 
 static void
@@ -963,11 +973,13 @@ peas_ui_plugin_manager_finalize (GObject *object)
  *
  * Creates a new plugin manager for the given #PeasEngine.
  *
- * Returns: the new #PeasUIPluginManager
+ * Returns: the new #PeasUIPluginManager.
  */
 GtkWidget *
 peas_ui_plugin_manager_new (PeasEngine *engine)
 {
+  g_return_val_if_fail (PEAS_IS_ENGINE (engine), NULL);
+
   return GTK_WIDGET (g_object_new (PEAS_UI_TYPE_PLUGIN_MANAGER,
                                    "engine", engine,
                                    NULL));
