@@ -20,13 +20,19 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "peas-extension-python.h"
 #include "peas-plugin-loader-python.h"
 
+/* _POSIX_C_SOURCE is defined in Python.h and in limits.h included by
+ * glib-object.h, so we unset it here to avoid a warning. Yep, that's bad. */
+#undef _POSIX_C_SOURCE
 #include <Python.h>
 #include <pygobject.h>
 #include <signal.h>
-#include "config.h"
 
 #if PY_VERSION_HEX < 0x02050000
 typedef int Py_ssize_t;
@@ -37,7 +43,9 @@ typedef int Py_ssize_t;
 struct _PeasPluginLoaderPythonPrivate {
   GHashTable *loaded_plugins;
   guint idle_gc;
-  gboolean init_failed;
+  guint init_failed : 1;
+  guint must_finalize_python : 1;
+  PyThreadState *py_thread_state;
 };
 
 typedef struct {
@@ -62,6 +70,7 @@ peas_register_types (PeasObjectModule *module)
                                               PEAS_TYPE_PLUGIN_LOADER_PYTHON);
 }
 
+/* NOTE: This must be called with the GIL held */
 static PyTypeObject *
 find_python_extension_type (PeasPluginInfo *info,
                             GType           exten_type,
@@ -77,20 +86,29 @@ find_python_extension_type (PeasPluginInfo *info,
   pytype = PyObject_GetAttrString (pygtype, "pytype");
   g_return_val_if_fail (pytype != NULL, NULL);
 
+  if (pytype == Py_None)
+    return NULL;
+
   while (PyDict_Next (locals, &pos, &key, &value))
     {
       if (!PyType_Check (value))
         continue;
 
-      if (PyObject_IsSubclass (value, pytype))
+      switch (PyObject_IsSubclass (value, pytype))
         {
+        case 1:
           Py_DECREF (pygtype);
           return (PyTypeObject *) value;
+        case 0:
+          continue;
+        case -1:
+          PyErr_Print ();
+          continue;
         }
     }
 
   Py_DECREF (pygtype);
-  g_debug ("No %s derivative found in Python plugin '%s'",
+  g_debug ("No '%s' derivative found in Python plugin '%s'",
            g_type_name (exten_type), peas_plugin_info_get_name (info));
   return NULL;
 }
@@ -102,13 +120,19 @@ peas_plugin_loader_python_provides_extension (PeasPluginLoader *loader,
 {
   PeasPluginLoaderPython *pyloader = PEAS_PLUGIN_LOADER_PYTHON (loader);
   PythonInfo *pyinfo;
+  PyTypeObject *extension_type;
+  PyGILState_STATE state;
 
   pyinfo = (PythonInfo *) g_hash_table_lookup (pyloader->priv->loaded_plugins, info);
 
   if (pyinfo == NULL)
     return FALSE;
 
-  return find_python_extension_type (info, exten_type, pyinfo->module) != NULL;
+  state = pyg_gil_state_ensure ();
+  extension_type = find_python_extension_type (info, exten_type, pyinfo->module);
+  pyg_gil_state_release (state);
+
+  return extension_type != NULL;
 }
 
 static PeasExtension *
@@ -123,16 +147,23 @@ peas_plugin_loader_python_get_extension (PeasPluginLoader *loader,
   PyGObject *pygobject;
   PyObject *emptyarg;
   PyObject *pyplinfo;
+  PyGILState_STATE state;
+  PeasExtension *exten;
 
   pyinfo = (PythonInfo *) g_hash_table_lookup (pyloader->priv->loaded_plugins, info);
 
   if (pyinfo == NULL)
     return NULL;
 
+  state = pyg_gil_state_ensure ();
+
   pytype = find_python_extension_type (info, exten_type, pyinfo->module);
 
   if (pytype == NULL || pytype->tp_new == NULL)
-    return NULL;
+    {
+      pyg_gil_state_release (state);
+      return NULL;
+    }
 
   emptyarg = PyTuple_New (0);
   pyobject = pytype->tp_new (pytype, emptyarg, NULL);
@@ -140,8 +171,10 @@ peas_plugin_loader_python_get_extension (PeasPluginLoader *loader,
 
   if (pyobject == NULL)
     {
-      g_error ("Could not create instance for %s.",
-               peas_plugin_info_get_name (info));
+      pyg_gil_state_release (state);
+      g_warning ("Could not create instance for '%s'",
+                 peas_plugin_info_get_name (info));
+
       return NULL;
     }
 
@@ -150,9 +183,10 @@ peas_plugin_loader_python_get_extension (PeasPluginLoader *loader,
   if (pygobject->obj != NULL)
     {
       Py_DECREF (pyobject);
-      g_error
-        ("Could not create instance for %s (GObject already initialized).",
-         peas_plugin_info_get_name (info));
+      pyg_gil_state_release (state);
+      g_warning ("Could not create instance for '%s': GObject already initialized",
+                 peas_plugin_info_get_name (info));
+
       return NULL;
     }
 
@@ -160,16 +194,19 @@ peas_plugin_loader_python_get_extension (PeasPluginLoader *loader,
 
   if (pygobject->obj == NULL)
     {
-      g_error ("Could not create %s instance for %s (GObject not constructed).",
-               g_type_name (exten_type), peas_plugin_info_get_name (info));
       Py_DECREF (pyobject);
+      pyg_gil_state_release (state);
+      g_warning ("Could not create '%s' instance for '%s': GObject not constructed",
+                 g_type_name (exten_type), peas_plugin_info_get_name (info));
 
       return NULL;
     }
 
+  g_return_val_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (pygobject->obj, exten_type), NULL);
+
   /* now call tp_init manually */
-  if (PyType_IsSubtype (pyobject->ob_type, pytype)
-      && pyobject->ob_type->tp_init != NULL)
+  if (PyType_IsSubtype (pyobject->ob_type, pytype) &&
+      pyobject->ob_type->tp_init != NULL)
     {
       emptyarg = PyTuple_New (0);
       pyobject->ob_type->tp_init (pyobject, emptyarg, NULL);
@@ -181,9 +218,13 @@ peas_plugin_loader_python_get_extension (PeasPluginLoader *loader,
   PyObject_SetAttrString (pyobject, "plugin_info", pyplinfo);
   Py_DECREF (pyplinfo);
 
-  return peas_extension_python_new (exten_type, pyobject);
+  exten = peas_extension_python_new (exten_type, pyobject);
+  pyg_gil_state_release (state);
+
+  return exten;
 }
 
+/* NOTE: This must be called with the GIL held */
 static void
 add_python_info (PeasPluginLoaderPython *loader,
                  PeasPluginInfo         *info,
@@ -203,9 +244,19 @@ peas_plugin_loader_python_add_module_directory (PeasPluginLoader *loader,
                                                 const gchar      *module_dir)
 {
   PeasPluginLoaderPython *pyloader = PEAS_PLUGIN_LOADER_PYTHON (loader);
+  PyGILState_STATE state;
 
-  g_debug ("Adding %s as a module path for the python loader.", module_dir);
+  /* Bail if we failed to initialise Python, since adding a module path won't
+   * help us, and calling the GIL functions will cause a crash. */
+  if (pyloader->priv->init_failed)
+    return;
+
+  state = pyg_gil_state_ensure ();
+
+  g_debug ("Adding '%s' as a module path for the Python loader", module_dir);
   peas_plugin_loader_python_add_module_path (pyloader, module_dir);
+
+  pyg_gil_state_release (state);
 }
 
 static gboolean
@@ -215,12 +266,14 @@ peas_plugin_loader_python_load (PeasPluginLoader *loader,
   PeasPluginLoaderPython *pyloader = PEAS_PLUGIN_LOADER_PYTHON (loader);
   PyObject *pymodule, *fromlist;
   gchar *module_name;
+  PyGILState_STATE state;
 
   if (pyloader->priv->init_failed)
     {
-      g_warning ("Cannot load python plugin Python '%s' since libpeas was "
-                 "not able to initialize the Python interpreter.",
+      g_warning ("Cannot load Python plugin '%s' since libpeas was "
+                 "not able to initialize the Python interpreter",
                  peas_plugin_info_get_name (info));
+
       return FALSE;
     }
 
@@ -228,6 +281,8 @@ peas_plugin_loader_python_load (PeasPluginLoader *loader,
   if (g_hash_table_lookup (pyloader->priv->loaded_plugins, info))
     return TRUE;
 
+  state = pyg_gil_state_ensure ();
+
   /* If we have a special path, we register it */
   peas_plugin_loader_python_add_module_path (pyloader,
                                              peas_plugin_info_get_module_dir (info));
@@ -245,11 +300,15 @@ peas_plugin_loader_python_load (PeasPluginLoader *loader,
     {
       g_free (module_name);
       PyErr_Print ();
+      pyg_gil_state_release (state);
+
       return FALSE;
     }
 
   add_python_info (pyloader, info, pymodule);
 
+  pyg_gil_state_release (state);
+
   return TRUE;
 }
 
@@ -273,11 +332,21 @@ peas_plugin_loader_python_unload (PeasPluginLoader *loader,
   pyinfo->module = NULL;
 }
 
+static void
+run_gc_protected (void)
+{
+  PyGILState_STATE state = pyg_gil_state_ensure ();
+
+  while (PyGC_Collect ())
+    ;
+
+  pyg_gil_state_release (state);
+}
+
 static gboolean
 run_gc (PeasPluginLoaderPython *loader)
 {
-  while (PyGC_Collect ())
-    ;
+  run_gc_protected ();
 
   loader->priv->idle_gc = 0;
   return FALSE;
@@ -297,9 +366,7 @@ peas_plugin_loader_python_garbage_collect (PeasPluginLoader *loader)
    * We both run the GC right now and we schedule
    * a further collection in the main loop.
    */
-
-  while (PyGC_Collect ())
-    ;
+  run_gc_protected ();
 
   if (pyloader->priv->idle_gc == 0)
     pyloader->priv->idle_gc = g_idle_add ((GSourceFunc) run_gc, pyloader);
@@ -311,22 +378,32 @@ peas_python_shutdown (PeasPluginLoaderPython *loader)
   if (!Py_IsInitialized ())
     return;
 
+  if (loader->priv->py_thread_state)
+    {
+      PyEval_RestoreThread (loader->priv->py_thread_state);
+      loader->priv->py_thread_state = NULL;
+    }
+
   if (loader->priv->idle_gc != 0)
     {
       g_source_remove (loader->priv->idle_gc);
       loader->priv->idle_gc = 0;
     }
 
-  while (PyGC_Collect ())
-    ;
+  run_gc_protected ();
 
-  Py_Finalize ();
+  if (loader->priv->must_finalize_python)
+    {
+      pyg_gil_state_ensure ();
+      Py_Finalize ();
+    }
 }
 
 /* C equivalent of
  *    import sys
  *    sys.path.insert(0, module_path)
  */
+/* NOTE: This must be called with the GIL held */
 static gboolean
 peas_plugin_loader_python_add_module_path (PeasPluginLoaderPython *self,
                                            const gchar            *module_path)
@@ -345,109 +422,76 @@ peas_plugin_loader_python_add_module_path (PeasPluginLoaderPython *self,
   if (PySequence_Contains (pathlist, pathstring) == 0)
     PyList_Insert (pathlist, 0, pathstring);
   Py_DECREF (pathstring);
-  return TRUE;
-}
 
-/* Note: the following function is needed because init_pyobject is a *macro*
- * which in case of error set the PyErr and then make the calling function
- * return behind our back.
- * It's up to the caller to check the result with PyErr_Occurred ()
- */
-static void
-peas_init_pygobject (void)
-{
-  init_pygobject_check (2, 11, 5);      /* FIXME: get from config */
+  return TRUE;
 }
 
 static gboolean
 peas_python_init (PeasPluginLoaderPython *loader)
 {
-  PyObject *mdict, *gobject, *gettext, *install, *gettext_args;
-  char *argv[] = { "libpeas", NULL };
-#ifdef HAVE_SIGACTION
-  struct sigaction old_sigint;
-  gint res;
-#endif
-
-  if (loader->priv->init_failed)
-    {
-      /* We already failed to initialized Python, don't need to
-       * retry again */
-      return FALSE;
-    }
-
-  if (Py_IsInitialized ())
-    {
-      /* Python has already been successfully initialized */
-      return TRUE;
-    }
+  PyObject *mdict, *gobject, *gi, *gettext, *install, *gettext_args;
+  char *argv[] = { "", NULL };
+  gchar *prgname;
 
   /* We are trying to initialize Python for the first time,
      set init_failed to FALSE only if the entire initialization process
      ends with success */
   loader->priv->init_failed = TRUE;
 
-  /* Hack to make python not overwrite SIGINT: this is needed to avoid
-   * the crash reported on bug #326191 */
-
-  /* CHECK: can't we use Py_InitializeEx instead of Py_Initialize in order
-     to avoid to manage signal handlers ? - Paolo (Dec. 31, 2006) */
-
-#ifdef HAVE_SIGACTION
-  /* Save old handler */
-  res = sigaction (SIGINT, NULL, &old_sigint);
-  if (res != 0)
-    {
-      g_warning ("Error initializing Python interpreter: cannot get "
-                 "handler to SIGINT signal (%s)", g_strerror (errno));
-
-      return FALSE;
-    }
-#endif
-
   /* Python initialization */
-  Py_Initialize ();
-
-#ifdef HAVE_SIGACTION
-  /* Restore old handler */
-  res = sigaction (SIGINT, &old_sigint, NULL);
-  if (res != 0)
+  if (!Py_IsInitialized ())
     {
-      g_warning ("Error initializing Python interpreter: cannot restore "
-                 "handler to SIGINT signal (%s).", g_strerror (errno));
-
-      goto python_init_error;
+      Py_InitializeEx (FALSE);
+      loader->priv->must_finalize_python = TRUE;
     }
-#endif
+
+  prgname = g_get_prgname ();
+  if (prgname != NULL)
+    argv[0] = prgname;
 
   PySys_SetArgv (1, argv);
 
+  /* Note that we don't call this with the GIL held, since we haven't initialised pygobject yet */
   peas_plugin_loader_python_add_module_path (loader, PEAS_PYEXECDIR);
 
   /* import gobject */
-  peas_init_pygobject ();
+  pygobject_init (PYGOBJECT_MAJOR_VERSION, PYGOBJECT_MINOR_VERSION, PYGOBJECT_MICRO_VERSION);
   if (PyErr_Occurred ())
     {
       g_warning ("Error initializing Python interpreter: could not "
-                 "import pygobject.");
+                 "import pygobject");
 
       goto python_init_error;
     }
 
+  /* Initialize support for threads */
+  pyg_enable_threads ();
+
   gobject = PyImport_ImportModule ("gobject");
   if (gobject == NULL)
     {
-      g_warning ("Error initializing Python interpreter: cound not "
-                 "import gobject.");
+      g_warning ("Error initializing Python interpreter: could not "
+                 "import gobject");
+
       goto python_init_error;
     }
 
   mdict = PyModule_GetDict (gobject);
   PyGObject_Type = PyDict_GetItemString (mdict, "GObject");
-  if (!PyGObject_Type)
+  if (PyGObject_Type == NULL)
     {
-      g_warning ("Error initializing Python interpreter: cound not "
+      g_warning ("Error initializing Python interpreter: could not "
                  "get gobject.GObject");
+
+      goto python_init_error;
+    }
+
+  gi = PyImport_ImportModule ("gi");
+  if (gi == NULL)
+    {
+      g_warning ("Error initializing Python interpreter: could not "
+                 "import gi");
+
       goto python_init_error;
     }
 
@@ -456,7 +500,7 @@ peas_python_init (PeasPluginLoaderPython *loader)
   if (gettext == NULL)
     {
       g_warning ("Error initializing Python interpreter: could not "
-                 "import gettext.");
+                 "import gettext");
 
       goto python_init_error;
     }
@@ -470,12 +514,14 @@ peas_python_init (PeasPluginLoaderPython *loader)
   /* Python has been successfully initialized */
   loader->priv->init_failed = FALSE;
 
+  loader->priv->py_thread_state = PyEval_SaveThread ();
+
   return TRUE;
 
 python_init_error:
 
   g_warning ("Please check the installation of all the Python related packages "
-             "required by libpeas and try again.");
+             "required by libpeas and try again");
 
   PyErr_Clear ();
 
@@ -537,8 +583,7 @@ peas_plugin_loader_python_class_init (PeasPluginLoaderPythonClass *klass)
   loader_class->provides_extension = peas_plugin_loader_python_provides_extension;
   loader_class->garbage_collect = peas_plugin_loader_python_garbage_collect;
 
-  g_type_class_add_private (object_class,
-                            sizeof (PeasPluginLoaderPythonPrivate));
+  g_type_class_add_private (object_class, sizeof (PeasPluginLoaderPythonPrivate));
 }
 
 static void
