@@ -1,7 +1,7 @@
 #include <gtk/gtk.h>
 #include <girepository.h>
-#include <libpeas/peas-engine.h>
-#include <libpeasui/peas-ui-plugin-manager.h>
+#include <libpeas/peas.h>
+#include <libpeasui/peas-ui.h>
 
 #include "peas-demo-window.h"
 
@@ -18,7 +18,7 @@ activate_plugin (GtkButton   *button,
   g_debug ("%s %s", G_STRFUNC, plugin_name);
   info = peas_engine_get_plugin_info (engine, plugin_name);
   g_return_if_fail (info != NULL);
-  peas_engine_activate_plugin (engine, info);
+  peas_engine_load_plugin (engine, info);
 }
 
 static void
@@ -90,7 +90,8 @@ create_main_window (void)
 }
 
 int
-main (int argc, char **argv)
+main (int    argc,
+      char **argv)
 {
   gchar const * const search_paths[] = {
     /* Uninstalled plugins */
